﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Drawing.Design;
using System.ComponentModel.Design;
using OPCClient;

namespace Bar
{
    /// <summary>
    /// Bar control class
    /// </summary>
    public partial class BarControl : UserControl
    {
        Pen pen = new Pen(Color.White, 0);
        BarControlConfig barControlConfig;
        OPCTagConfig opcTagConfig = new OPCTagConfig();
        OPCConfig opcConfig;

        /// <summary>
        /// Default constructor of the bar control class. Creates new bar control.
        /// </summary>
        public BarControl()
        {
            InitializeComponent();
        }

        //Bar control atributes.
        //You cannot access them outside this class. 
        //Instead, use their corresponding property methods.
        private Color barColor = Color.Empty;
        private Color backColor = Color.Empty;
        private double barValue = 0.0;
        private double minBarValue;
        private double maxBarValue;
        private double tankCapacity = 100.0;

        /// <summary>
        /// Bar color
        /// </summary>
        /// <remarks>
        /// Use the <b>BarColor</b> property to specify the color of dynamic part of the bar control.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Bar Control Properties"), Description("Represents color of the dinamic part of bar control")]
        public Color BarColor
        {
            get
            {
                return barColor;
            }
            set
            {
                barColor = value;

                //This method is used for triggering onPaint(PaintEventArgs) event method so that a bar control could be redrawn.
                //If the value is set in this property, Invalidate() method will be called.
                this.Invalidate(); 
            }
        }

        /// <summary>
        /// Background color
        /// </summary>
        /// <remarks>
        /// Use the <b>BackColor1</b> property to specify the background color of bar control.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Bar Control Properties"), Description("Background color of bar control")]
        public Color BackColor1
        {
            get
            {
                return backColor;
            }
            set
            {
                backColor = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Bar value
        /// </summary>
        /// <remarks>
        /// Use the <b>BarValue</b> property to set a numeric value that is responsible for changing the dinamic part of the bar control.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Bar Control Properties"), 
        Description("Numeric value that is responsible for bar control state. This value is responsible for changing the dinamic part of the bar control")]
        public double BarValue
        {
            get 
            { 
                return barValue; 
            }
            set
            {
                barValue = value;
                this.Invalidate();          
            }
        }

        /// <summary>
        /// Minimal allowed bar value
        /// </summary>
        /// <remarks>
        /// Use the <b>MinBarValue</b> property to set a minimal bar value that is allowed. If the bar value is less than minimal limit, alarm indicator will appear.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Bar Control Properties"), Description("Minimal bar value that is allowed. If the bar value is less than minimal limit, alarm indicator will appear")]
        public double MinBarValue
        {
            get
            {
                return minBarValue;
            }
            set
            {
                minBarValue = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Maximal allowed bar value
        /// </summary>
        /// <remarks>
        /// Use the <b>MaxBarValue</b> property to set a maximal bar value that is allowed. If the bar value is greater than maximal limit, alarm indicator will appear.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Bar Control Properties"), Description("Maximal bar value that is allowed. If the bar value is greater than maximal limit, alarm indicator will appear")]
        public double MaxBarValue
        {
            get
            {
                return maxBarValue;
            }
            set
            {
                maxBarValue = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Capacity of the tank that contains bar control
        /// </summary>
        /// <remarks>
        /// Use the <b>TankCapacity</b> property to set desired capacity of the tank.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Bar Control Properties"), Description("Represents capacity of the tank that contains bar control")]
        public double TankCapacity
        {
            get
            {
                return tankCapacity;
            }
            set
            {
                tankCapacity = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Visibility of the bar control
        /// </summary>
        /// <remarks>
        /// Use the <b>BarControlVisibility</b> property to set whether a bar control is visible or not.
        /// This property can be set using Properties window in design mode only.
        /// </remarks>
        [Category("Bar Control Properties"), Description("Property that is used for changing the visibility of the bar control")]
        public bool BarControlVisibility
        {
            get
            {
                return this.Visible;
            }
            set
            {
                this.Visible = value;
            }
        }

        /// <summary>
        /// Starting or stopping bar control
        /// </summary>
        /// <remarks>
        /// Use the <b>StartStopBarControl</b> property to start or stop timer that servers like a trigger for starting and stopping bar control.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Bar Control Properties"), Description("Timer property that serves like a trigger for starting and stopping bar control")]
        public Timer StartStopBarControl
        {
            get
            {
                return refreshTimer;
            }
            set
            {
                refreshTimer = value;
            }
        }

        /// <summary>
        /// OPC Configuration class
        /// </summary>
        /// <remarks>
        /// Use the <b>OpcTagConfig</b> property to set configured opcTagConfig object that contains host, group, tag and tagValue property.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Bar Control Properties"), Description("OPC tag configuration")]
        [Editor(typeof(BarControlOPCEditor), typeof(UITypeEditor))]
        public OPCTagConfig OpcTagConfig
        {
            get { return opcTagConfig; }
            set
            {
                opcTagConfig = value;
                this.Invalidate();
            }
        }

        //overriding existing OnPaint()event method that is modified and used for redrawing a bar control.
        //this method is triggered by Invalidate() method.
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

            Brush backColorBrush = new SolidBrush(backColor);
            Brush barColorBrush = new SolidBrush(barColor);

            Rectangle rect = new Rectangle(38, 36, barPictureBox.Width, barPictureBox.Height);

            firstMeasureLabel.Text = "- 0";
            secondMeasureLabel.Text = "- " + (Convert.ToInt32(this.TankCapacity * 0.25)).ToString();
            thirdMeasureLabel.Text = "- " + (Convert.ToInt32(this.TankCapacity * 0.50)).ToString();
            fourthMeasureLabel.Text = "- " + (Convert.ToInt32(this.TankCapacity * 0.75)).ToString();
            fifthMeasureLabel.Text = "- " + (Convert.ToInt32(this.TankCapacity)).ToString();

            if ((barValue > minBarValue) || (barValue < maxBarValue))
            {
                dozvoljenaVrijednostPictureBox.Show();
                nedozvoljenaVrijednostPictureBox.Hide();
            }

            if ((barValue < minBarValue) || (barValue > maxBarValue))
            {
                dozvoljenaVrijednostPictureBox.Hide();
                nedozvoljenaVrijednostPictureBox.Show();
            }

            e.Graphics.DrawRectangle(pen, rect);
            e.Graphics.FillRectangle(backColorBrush, rect);

            int height = (int)(((barValue / this.TankCapacity) * barPictureBox.Height));
            barControlValueLabel.Text = barValue.ToString();

            try
            {
                e.Graphics.FillRectangle(barColorBrush, 38, barPictureBox.Height - height + 36, barPictureBox.Width, height);
            }
            catch (OverflowException over)
            {
                over = new OverflowException();
            }
            barPictureBox.Hide();
        }

        private void oPCTagConfigToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            opcConfig = new OPCConfig();
            opcConfig.ShowDialog();
            applyOPCConfigToolStripMenuItem.Enabled = true;
            turnOnBarGraphToolStripMenuItem.Enabled = true;
        }

        private void applyOPCConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.OpcTagConfig = opcConfig.opcTagConfig;
            applyOPCConfigToolStripMenuItem.Enabled = false;
        }

        private void barControlConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            barControlConfig = new BarControlConfig();
            barControlConfig.ShowDialog();
            applyBarConfigToolStripMenuItem.Enabled = true;
        }

        private void applyBarConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.BackColor1 = barControlConfig.backgroundColorPictureBox.BackColor;
            this.BarColor = barControlConfig.barColorPictureBox.BackColor;
            this.TankCapacity = barControlConfig.TempCapacity;
            this.MaxBarValue = barControlConfig.TempMaxBarValue;
            this.MinBarValue = barControlConfig.TempMinBarValue;
            this.StartStopBarControl.Interval = barControlConfig.RefreshTime;
            applyBarConfigToolStripMenuItem.Enabled = false;
        }

        private void turnOnBarGraphToolStripMenuItem_Click(object sender, EventArgs e)
        {
            refreshTimer.Start();
            turnOnBarGraphToolStripMenuItem.Enabled = false;
            turnOffBarGraphToolStripMenuItem.Enabled = true;
        }

        private void turnOffBarGraphToolStripMenuItem_Click(object sender, EventArgs e)
        {
            refreshTimer.Stop();
            turnOnBarGraphToolStripMenuItem.Enabled = true;
            turnOffBarGraphToolStripMenuItem.Enabled = false;
        }

        private void refreshTimer_Tick(object sender, EventArgs e)
        {
            this.BarValue = Convert.ToDouble
            (opcSystemsComponent1.GetTag_Parameter_Value("Value", "Value", opcTagConfig.Tag, opcTagConfig.Group, opcTagConfig.Host));
        }
    }
}
