﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using OPCSystems;

namespace Bar
{
    /// <summary>
    /// Bar control configuration class
    /// </summary>
    public partial class BarControlConfig : Form
    {
        /// <summary>
        /// Default constructor of the BarControlConfig class. Creates new BarControlConfig form.
        /// </summary>
        public BarControlConfig()
        {
            InitializeComponent();
        }

        //Local atributes that are used for passing a values of those atributes to corresponding atributes on bar control.
        //You cannot access them outside this class. 
        //Instead, use their corresponding property methods.
        private double tempCapacity;
        private double tempMinBarValue;
        private double tempMaxBarValue;

        /// <summary>
        /// Temporary capacity property that stores choosen tank capacity value for bar control
        /// </summary>
        /// <remarks>
        /// This property is used only for passing a value to bar controls.
        /// </remarks>
        public double TempCapacity
        {
            get { return tempCapacity; }
            protected set { tempCapacity = value; }
        }

        /// <summary>
        /// Temporary min bar property that stores choosen minimum allowed bar value for bar control
        /// </summary>
        /// <remarks>
        /// This property is used only for passing a value to bar controls.
        /// </remarks>
        public double TempMinBarValue
        {
            get { return tempMinBarValue; }
            protected set { tempMinBarValue = value; }
        }

        /// <summary>
        /// Temporary max bar property that stores choosen maximum allowed bar value for bar control
        /// </summary>
        /// <remarks>
        /// This property is used only for passing a value to bar controls.
        /// </remarks>
        public double TempMaxBarValue
        {
            get { return tempMaxBarValue; }
            protected set { tempMaxBarValue = value; }
        }

        /// <summary>
        /// Refresh time of the bar control
        /// </summary>
        /// <remarks>
        /// This property is used for setting a refresh time of the bar control.
        /// </remarks>
        public int RefreshTime
        {
            get { return refreshTimer.Interval; }
            protected set { refreshTimer.Interval = value; }
        }

        #region bar graph color configuration

        private void chooseBackgroundButton_Click(object sender, EventArgs e)
        {
            colorDialog1.ShowDialog();
            backgroundColorPictureBox.BackColor = colorDialog1.Color;
            EnableOtherOptions();
        }

        private void chooseBarColorButton_Click(object sender, EventArgs e)
        {
            colorDialog1.ShowDialog();
            barColorPictureBox.BackColor = colorDialog1.Color;
            EnableOtherOptions();
        }

        private void EnableOtherOptions()
        {
            if ((backgroundColorPictureBox.BackColor != Color.Empty) && (barColorPictureBox.BackColor != Color.Empty))
            {
                tankCapacityConfigurationGroupBox.Enabled = true;
            }
        }

        #endregion


        #region tank capacity configuration

        private void tankCapacityButton_Click(object sender, EventArgs e)
        {
            try
            {
                this.TempCapacity = Convert.ToDouble(tankCapacityTextBox.Text);
                alarmConfigurationGroupBox.Enabled = true;
                RemoveErrorDescriptionText();
            }
            catch (FormatException formatError)
            {
                problemLabel.Text = "Tank capacity is not configured";
                formatError = new FormatException("Tank capacity is not entered in a correct format");
                causeOfProblemLabel.Text = formatError.Message;
                alarmConfigurationGroupBox.Enabled = false;
            }
        }

        #endregion


        #region minimal and maximal permitted value configuration

        private void alarmConfigurationButton_Click(object sender, EventArgs e)
        {
            try
            {
                this.TempMinBarValue = Convert.ToDouble(topLimitTextBox.Text);
                this.TempMaxBarValue = Convert.ToDouble(bottomLimitTextBox.Text);
                okButton.Enabled = true;
                RemoveErrorDescriptionText();

                if ((this.TempMaxBarValue < 0) || (this.TempMaxBarValue > this.TempCapacity))
                {
                    throw new ArgumentException();
                }

                if ((this.TempMinBarValue < 0) || (this.TempMinBarValue > this.TempCapacity))
                {
                    throw new ArgumentException();
                }

                if (this.TempMinBarValue >= this.TempMaxBarValue)
                {
                    throw new Exception("Bottom limit value cant be larger than top limit value");
                }
            }
            catch (ArgumentException ex1)
            {
                problemLabel.Text = "Indicator of permitted value is not configured";
                ex1 = new ArgumentException("Limit values cant be less than 0 or larger than tank capacity");
                causeOfProblemLabel.Text = ex1.Message;
                okButton.Enabled = false;
            }
            catch (FormatException formatError2)
            {
                problemLabel.Text = "Indicator of permitted value is not configured";
                formatError2 = new FormatException("Limit values are not entered in a correct format");
                causeOfProblemLabel.Text = formatError2.Message;
                okButton.Enabled = false;
            }
            catch (Exception ex2)
            {
                problemLabel.Text = "Indicator of permitted value is not configured";
                causeOfProblemLabel.Text = ex2.Message;
                okButton.Enabled = false;
            }
        }

        #endregion


        #region refresh time of the control (miliseconds) - default value is 100

        private void refreshTimeConfigurationButton_Click(object sender, EventArgs e)
        {
            try
            {
                this.RefreshTime = int.Parse(refreshTimeTextBox.Text);
            }
            catch (FormatException formatError3)
            {
                problemLabel.Text = "Indicator of permitted value is not configured";
                formatError3 = new FormatException("Refresh time value is not entered in a correct format");
                causeOfProblemLabel.Text = formatError3.Message;
            }
        }

        #endregion

        private void RemoveErrorDescriptionText()
        {
            problemLabel.Text = "";
            causeOfProblemLabel.Text = "";
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
        }
    }
}
