﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Drawing.Design;
using System.ComponentModel.Design;
using OPCClient;

namespace Pipeline
{
    /// <summary>
    /// Horizontal pipe control class
    /// </summary>
    public partial class HorizontalPipeControl : UserControl
    {
        PipelineControlConfig pipelineControlConfig;
        OPCConfig opcConfig;
        OPCTagConfig opcTagConfig = new OPCTagConfig();
        Pen pen = new Pen(Color.White);

        /// <summary>
        /// Default constructor of the horizontal pipe control class. Creates new horizontal pipe control.
        /// </summary>
        public HorizontalPipeControl()
        {
            InitializeComponent();
        }

        //Horizontal pipe control atributes.
        //You cannot access them outside this class. 
        //Instead, use their corresponding property methods.
        private bool fillPipe = false;
        private Color horizontalPipeColor = Color.Empty;
        private double horizontalPipeValue = 0.0;

        /// <summary>
        /// Status of the horizontal pipe control
        /// </summary>
        /// <remarks>
        /// Use the <b>FillPipe</b> property to specify the status of the horizontal pipe control. If property value is false, horizontal pipe is empty. Otherwise, horizontal pipe is filled with liquid.
        /// This property can be set using Properties window in design mode. It can also be used in development for checking and setting the status of the horizontal pipe control.
        /// </remarks>
        [Category("Pipeline Control Properties"), Description("Status of the horizontal pipe control")]
        public bool FillPipe
        {
            get { return fillPipe; }
            set
            {
                fillPipe = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Background color
        /// </summary>
        /// <remarks>
        /// Use the <b>BackColor1</b> property to specify the background color that will appear in horizontal pipe control.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Pipeline Control Properties"), Description("Background color of the horizontal pipe control")]
        public Color BackColor1
        {
            get
            {
                return horizontalPipeColor;
            }
            set
            {
                horizontalPipeColor = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Horizontal pipe value
        /// </summary>
        /// <remarks>
        /// Use the <b>HorizontalPipeValue</b> property to set a numeric value that decides whether a horizontal pipe control is filled or empty.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Pipeline Control Properties"),
        Description("Numeric value that is responsible for horizontal pipe control state. If this value is greater than 0, pipe will be filled with choosen BackColor1 property. Otherwise, pipe will be empty")]
        public double HorizontalPipeValue
        {
            get
            {
                return horizontalPipeValue;
            }
            set
            {
                horizontalPipeValue = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Visibility of the horizontal pipe control
        /// </summary>
        /// <remarks>
        /// Use the <b>PipeControlVisibility</b> property to set whether a horizontal pipe control is visible or not.
        /// This property can be set using Properties window in design mode only.
        /// </remarks>
        [Category("Pipeline Control Properties"), Description("Property that is used for changing the visibility of the horizontal pipe control")]
        public bool PipeControlVisibility
        {
            get
            {
                return this.Visible;
            }
            set
            {
                this.Visible = value;
            }
        }

        /// <summary>
        /// Starting or stopping horizontal pipe control
        /// </summary>
        /// <remarks>
        /// Use the <b>StartStopPipeControl</b> property to start or stop timer that servers like a trigger for starting and stopping horizontal pipe control.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Pipeline Control Properties"), Description("Timer property that serves like a trigger for starting and stopping horizontal pipe control")]
        public Timer StartStopPipeControl
        {
            get 
            { 
                return refreshTimer; 
            }
            set 
            { 
                refreshTimer = value; 
            }
        }

        /// <summary>
        /// OPC Configuration class
        /// </summary>
        /// <remarks>
        /// Use the <b>OpcTagConfig</b> property to set configured opcTagConfig object that contains host, group, tag and tagValue property.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Pipeline Control Properties"), Description("OPC tag configuration")]
        [Editor(typeof(PipelineControlOPCEditor), typeof(UITypeEditor))]
        public OPCTagConfig OpcTagConfig
        {
            get { return opcTagConfig; }
            set
            {
                opcTagConfig = value;
                this.Invalidate();
            }
        }
       

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            Brush backColorBrush;
            Rectangle horizontalRect = new Rectangle(0, 9, horizontalPipePictureBox.Width, horizontalPipePictureBox.Height);
            e.Graphics.DrawRectangle(pen, horizontalRect);

            if ((fillPipe == true) || (this.HorizontalPipeValue > 0.0))
            {
                backColorBrush = new SolidBrush(this.BackColor1);
                e.Graphics.FillRectangle(backColorBrush, horizontalRect);
            }
            else
            {
                backColorBrush = new SolidBrush(Color.Empty);
                e.Graphics.FillRectangle(backColorBrush, horizontalRect);
            }

            horizontalPipePictureBox.Hide();
            backColorBrush.Dispose();
        }

        private void oPCTagConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            opcConfig = new OPCConfig();
            opcConfig.ShowDialog();
            applyOPCConfigToolStripMenuItem.Enabled = true;
            turnOnPipeToolStripMenuItem.Enabled = true;
        }

        private void applyOPCConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.OpcTagConfig = opcConfig.opcTagConfig;
            applyOPCConfigToolStripMenuItem.Enabled = false;
        }

        private void pipeControlConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            pipelineControlConfig = new PipelineControlConfig();
            pipelineControlConfig.ShowDialog();
            applyPipeConfigToolStripMenuItem.Enabled = true;
        }

        private void applyPipeConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.BackColor1 = pipelineControlConfig.pipeColorPictureBox.BackColor;
            this.StartStopPipeControl.Interval = pipelineControlConfig.RefreshTime;
            applyPipeConfigToolStripMenuItem.Enabled = false;
        }

        private void turnOnPipeToolStripMenuItem_Click_1(object sender, EventArgs e)
        {
            refreshTimer.Start();
            turnOnPipeToolStripMenuItem.Enabled = false;
            turnOffPipeToolStripMenuItem.Enabled = true;
        }

        private void turnOffPipeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            refreshTimer.Stop();
            turnOnPipeToolStripMenuItem.Enabled = true;
            turnOffPipeToolStripMenuItem.Enabled = false;
        }

        private void refreshTimer_Tick(object sender, EventArgs e)
        {
            this.HorizontalPipeValue = Convert.ToDouble
            (opcSystemsComponent1.GetTag_Parameter_Value("Value", "Value", opcTagConfig.Tag, opcTagConfig.Group, opcTagConfig.Host));
        }
    }
}
