﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Drawing.Design;
using System.ComponentModel.Design;
using OPCClient;

namespace Pump
{
    /// <summary>
    /// Vertical pump control class
    /// </summary>
    public partial class VerticalPumpControl : UserControl
    {
        OPCConfig opcConfig;
        PumpControlConfig pumpControlConfig;
        OPCTagConfig opcTagConfig = new OPCTagConfig();

        /// <summary>
        /// Default constructor of the vertical pump control class. Creates new vertical pump control.
        /// </summary>
        public VerticalPumpControl()
        {
            InitializeComponent();
            manualControlToolStripMenuItem.Checked = true;
        }

        //Vertical pump control atributes.
        //You cannot access them outside this class. 
        //Instead, use their corresponding property methods.
        private bool startPump = false;
        private Timer time;
        private double pumpValue = 0.0;
        private double pumpLimitValue = 0.0;

        /// <summary>
        /// Status of the vertical pump control
        /// </summary>
        /// <remarks>
        /// Use the <b>StartPump</b> property to specify the status of the vertical pump control. If property value is false, vertical pump is stopped. Otherwise, vertical pump is started.
        /// This property can be set using Properties window in design mode. It can also be used in development for checking and setting the status of the vertical pump control.
        /// </remarks>
        [Category("Pump Control Properties"), Description("Property that is used for checking and setting the status of the vertical pump control")]
        public bool StartPump
        {
            get { return startPump; }
            set 
            { 
                startPump = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Internal timer
        /// </summary>
        /// <remarks>
        /// Use the <b>Time</b> property for setting the reference from another timer object so you could control timer event using vertical pump control.
        /// This property is usually used only in development mode. 
        /// </remarks>
        [Category("Pump Control Properties"), Description("Internal timer that is used for developing custom application using vertical pump control. You can pass the reference of another timer object to this property")]
        public Timer Time
        {
            get { return time; }
            set { time = value; }
        }

        /// <summary>
        /// Vertical pump value
        /// </summary>
        /// <remarks>
        /// Use the <b>ValveValue</b> property to set a numeric value that decides whether a vertical pump control is started or stopped.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Pump Control Properties"), Description("Numeric value that is responsible for vertical pump control state. If this value is less than PumpLimitValue, pump will be turned off. Otherwise, pump will be turned on")]
        public double PumpValue
        {
            get
            {
                return pumpValue;
            }
            set
            {
                pumpValue = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Vertical pump limit value
        /// </summary>
        /// <remarks>
        /// Use the <b>ValveLimitValue</b> property to set a limit value that decides when the vertical pump control will be started or stopped.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Pump Control Properties"), Description("Limit value decides when the vertical pump control will be open or closed. Default pump limit value is 0.0")]
        public double PumpLimitValue
        {
            get
            {
                return pumpLimitValue;
            }
            set
            {
                pumpLimitValue = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Visibility of the vertical pump control
        /// </summary>
        /// <remarks>
        /// Use the <b>ValveControlVisibility</b> property to set whether a vertical pump control is visible or not.
        /// This property can be set using Properties window in design mode only.
        /// </remarks>
        [Category("Pump Control Properties"), Description("Property that is used for changing the visibility of the vertical pump control")]
        public bool PumpControlVisibility
        {
            get
            {
                return this.Visible;
            }
            set
            {
                this.Visible = value;
            }
        }

        /// <summary>
        /// Starting or stopping vertical pump control
        /// </summary>
        /// <remarks>
        /// Use the <b>StartStopVerticalPumpControl</b> property to start or stop timer that servers like a trigger for starting and stopping vertical pump control.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Pump Control Properties"), Description("Timer property that serves like a trigger for starting and stopping vertical pump control")]
        public Timer StartStopVerticalPumpControl
        {
            get
            {
                return refreshTimer;
            }
            set
            {
                refreshTimer = value;
            }
        }

        /// <summary>
        /// OPC Configuration class
        /// </summary>
        /// <remarks>
        /// Use the <b>OpcTagConfig</b> property to set configured opcTagConfig object that contains host, group, tag and tagValue property.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Pump Control Properties"), Description("OPC tag configuration")]
        [Editor(typeof(PumpControlOPCEditor), typeof(UITypeEditor))]
        public OPCTagConfig OpcTagConfig
        {
            get { return opcTagConfig; }
            set
            {
                opcTagConfig = value;
                this.Invalidate();
            }
        }


        //Following event methods are used for automatic control of the pump
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            if ((startPump == true) || (this.PumpValue > this.PumpLimitValue))
            {
                verticalPumpStartPictureBox.Show();
                verticalPumpStopPictureBox.Hide();
            }
            else
            {
                verticalPumpStopPictureBox.Show();
                verticalPumpStartPictureBox.Hide();
            }
        }

        private void oPCTagConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            opcConfig = new OPCConfig();
            opcConfig.ShowDialog();
            applyOPCConfigToolStripMenuItem.Enabled = true;
            turnOnPumpToolStripMenuItem.Enabled = true;
        }

        private void applyOPCConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.OpcTagConfig = opcConfig.opcTagConfig;
            applyOPCConfigToolStripMenuItem.Enabled = false;
            manualControlToolStripMenuItem.Checked = false;
            automaticControlToolStripMenuItem.Checked = true;
        }

        private void pumpControlConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            pumpControlConfig = new PumpControlConfig();
            pumpControlConfig.ShowDialog();
            applyPumpConfigToolStripMenuItem.Enabled = true;
        }

        private void applyPumpConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.PumpLimitValue = pumpControlConfig.TempLimitValue;
            this.StartStopVerticalPumpControl.Interval = pumpControlConfig.RefreshTime;
            applyPumpConfigToolStripMenuItem.Enabled = false;
        }

        private void turnOnPumpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            refreshTimer.Start();
            turnOnPumpToolStripMenuItem.Enabled = false;
            turnOffPumpToolStripMenuItem.Enabled = true;
        }

        private void turnOffPumpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            refreshTimer.Stop();
            turnOnPumpToolStripMenuItem.Enabled = true;
            turnOffPumpToolStripMenuItem.Enabled = false;
            verticalPumpStartPictureBox.Hide();
            verticalPumpStopPictureBox.Show();
        }

        private void refreshTimer_Tick(object sender, EventArgs e)
        {
            this.PumpValue = Convert.ToDouble
            (opcSystemsComponent1.GetTag_Parameter_Value("Value", "Value", opcTagConfig.Tag, opcTagConfig.Group, opcTagConfig.Host));
        }

        //Following event methods are used for manual control of the pump
        private void verticalPumpStopPictureBox_Click(object sender, EventArgs e)
        {
            startPump = true;
            verticalPumpStartPictureBox.Show();
            verticalPumpStopPictureBox.Hide();
            try
            {
                this.Time.Start();
            }
            catch (NullReferenceException nre1)
            {
                nre1 = new NullReferenceException();
            }
        }

        private void verticalPumpStartPictureBox_Click(object sender, EventArgs e)
        {
            startPump = false;
            verticalPumpStopPictureBox.Show();
            verticalPumpStartPictureBox.Hide();
            try
            {
                this.Time.Stop();
            }
            catch (NullReferenceException nre2)
            {
                nre2 = new NullReferenceException();
            }
        }

        private void applyManualControlToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            refreshTimer.Enabled = false;
            applyOPCConfigToolStripMenuItem.Enabled = false;
            applyPumpConfigToolStripMenuItem.Enabled = false;
            turnOnPumpToolStripMenuItem.Enabled = false;
            turnOffPumpToolStripMenuItem.Enabled = false;
            manualControlToolStripMenuItem.Checked = true;
            automaticControlToolStripMenuItem.Checked = false;
        }
    }
}
