﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Drawing.Design;
using System.ComponentModel.Design;
using OPCClient;

namespace Valve
{
    /// <summary>
    /// Vertical valve control class
    /// </summary>
    public partial class VerticalValveControl : UserControl
    {
        OPCConfig opcConfig;
        ValveControlConfig valveControlConfig;
        OPCTagConfig opcTagConfig = new OPCTagConfig();

        /// <summary>
        /// Default constructor of the vertical valve control class. Creates new vertical valve control.
        /// </summary>
        public VerticalValveControl()
        {
            InitializeComponent();
            manualControlToolStripMenuItem.Checked = true;
        }

        //Vertical valve control atributes.
        //You cannot access them outside this class. 
        //Instead, use their corresponding property methods.
        private bool openValve = false;
        private double valveValue = 0.0;
        private double valveLimitValue = 0.0;

        /// <summary>
        /// Status of the vertical valve control
        /// </summary>
        /// <remarks>
        /// Use the <b>OpenValve</b> property to specify the status of the vertical valve control. If property value is false, vertical valve is closed. Otherwise, vertical valve is open.
        /// This property can be set using Properties window in design mode. It can also be used in development for checking and setting the status of the vertical valve control.
        /// </remarks>
        [Category("Valve Control Properties"), Description("Property that is used for checking and setting the status of the vertical valve control")]
        public bool OpenValve
        {
            get { return openValve; }
            set 
            { 
                openValve = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Vertical valve value
        /// </summary>
        /// <remarks>
        /// Use the <b>ValveValue</b> property to set a numeric value that decides whether a vertical valve control is open or closed.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Valve Control Properties"), Description("Numeric value that is responsible for vertical valve control state. If this value is less than ValveLimitValue, valve will be turned off. Otherwise, valve will be turned on")]
        public double ValveValue
        {
            get
            {
                return valveValue;
            }
            set
            {
                valveValue = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Vertical valve limit value
        /// </summary>
        /// <remarks>
        /// Use the <b>ValveLimitValue</b> property to set a limit value that decides when the vertical valve control will be open or closed.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Valve Control Properties"), Description("Limit value decides when the vertical valve control will be open or closed. Default valve limit value is 0.0")]
        public double ValveLimitValue
        {
            get
            {
                return valveLimitValue;
            }
            set
            {
                valveLimitValue = value;
                this.Invalidate();
            }
        }

        /// <summary>
        /// Visibility of the vertical valve control
        /// </summary>
        /// <remarks>
        /// Use the <b>ValveControlVisibility</b> property to set whether a vertical valve control is visible or not.
        /// This property can be set using Properties window in design mode only.
        /// </remarks>
        [Category("Valve Control Properties"), Description("Property that is used for changing the visibility of the vertical valve control")]
        public bool ValveControlVisibility
        {
            get
            {
                return this.Visible;
            }
            set
            {
                this.Visible = value;
            }
        }

        /// <summary>
        /// Starting or stopping vertical valve control
        /// </summary>
        /// <remarks>
        /// Use the <b>StartStopVerticalValveControl</b> property to start or stop timer that servers like a trigger for starting and stopping vertical valve control.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Valve Control Properties"), Description("Timer property that serves like a trigger for starting and stopping vertical valve control")]
        public Timer StartStopVerticalValveControl
        {
            get
            {
                return refreshTimer;
            }
            set
            {
                refreshTimer = value;
            }
        }

        /// <summary>
        /// OPC Configuration class
        /// </summary>
        /// <remarks>
        /// Use the <b>OpcTagConfig</b> property to set configured opcTagConfig object that contains host, group, tag and tagValue property.
        /// This property can be set using Properties window in design mode, or using context menu in runtime mode.
        /// </remarks>
        [Category("Valve Control Properties"), Description("OPC tag configuration")]
        [Editor(typeof(ValveControlOPCEditor), typeof(UITypeEditor))]
        public OPCTagConfig OpcTagConfig
        {
            get { return opcTagConfig; }
            set
            {
                opcTagConfig = value;
                this.Invalidate();
            }
        }


        //Following event methods are used for automatic control of the valve
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            if ((openValve == true) || (this.ValveValue > this.ValveLimitValue))
            {
                verticalValveStartPictureBox.Show();
                verticalValveStopPictureBox.Hide();
            }
            else
            {
                verticalValveStopPictureBox.Show();
                verticalValveStartPictureBox.Hide();
            }
        }

        private void oPCTagConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            opcConfig = new OPCConfig();
            opcConfig.ShowDialog();
            applyOPCConfigToolStripMenuItem.Enabled = true;
            turnOnValveToolStripMenuItem.Enabled = true;
        }

        private void applyOPCConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.OpcTagConfig = opcConfig.opcTagConfig;
            applyOPCConfigToolStripMenuItem.Enabled = false;
            manualControlToolStripMenuItem.Checked = false;
            automaticControlToolStripMenuItem.Checked = true;
        }

        private void valveControlConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            valveControlConfig = new ValveControlConfig();
            valveControlConfig.ShowDialog();
            applyValveConfigToolStripMenuItem.Enabled = true;
        }

        private void applyValveConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.ValveLimitValue = valveControlConfig.TempLimitValue;
            this.StartStopVerticalValveControl.Interval = valveControlConfig.RefreshTime;
            applyValveConfigToolStripMenuItem.Enabled = false;
        }

        private void turnOnValveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            refreshTimer.Start();
            turnOnValveToolStripMenuItem.Enabled = false;
            turnOffValveToolStripMenuItem.Enabled = true;
        }

        private void turnOffValveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            refreshTimer.Stop();
            turnOnValveToolStripMenuItem.Enabled = true;
            turnOffValveToolStripMenuItem.Enabled = false;
            verticalValveStopPictureBox.Show();
            verticalValveStartPictureBox.Hide();
        }

        private void refreshTimer_Tick(object sender, EventArgs e)
        {
            this.ValveValue = Convert.ToDouble
            (opcSystemsComponent1.GetTag_Parameter_Value("Value", "Value", opcTagConfig.Tag, opcTagConfig.Group, opcTagConfig.Host));
        }

        //Following event methods are used for manual control of the valve
        private void verticalValveStopPictureBox_Click(object sender, EventArgs e)
        {
            openValve = true;
            verticalValveStartPictureBox.Show();
            verticalValveStopPictureBox.Hide();
        }

        private void verticalValveStartPictureBox_Click(object sender, EventArgs e)
        {
            openValve = false;
            verticalValveStopPictureBox.Show();
            verticalValveStartPictureBox.Hide();
        }

        private void applyManualControlToolStripMenuItem_Click(object sender, EventArgs e)
        {
            refreshTimer.Dispose();
            applyOPCConfigToolStripMenuItem.Enabled = false;
            applyValveConfigToolStripMenuItem.Enabled = false;
            turnOnValveToolStripMenuItem.Enabled = false;
            turnOffValveToolStripMenuItem.Enabled = false;
            manualControlToolStripMenuItem.Checked = true;
            automaticControlToolStripMenuItem.Checked = false;
        }
    }
}
